﻿-- Author      : Alistair
-- Create Date : 7/23/2008 7:44:38 PM

--global for frame locking, always locked on loading
AURAUNITFRAMES_FRAME_FRAMESLOCKED = true
AURAUNITFRAMES_SHOW_PETS = true

local AUF_Loaded = false

local AUF_Events = {"UNIT_HEALTH", "UNIT_MAXHEALTH",
							"UNIT_MANA", "UNIT_MAXMANA",
							"UNIT_ENERGY", "UNIT_MAXENERGY", 
							"UNIT_RAGE", "UNIT_MAXRAGE",
							"PLAYER_ENTERING_WORLD", "ADDON_LOADED",
							"PLAYER_TARGET_CHANGED", "UNIT_DISPLAYPOWER",
							"PLAYER_FOCUS_CHANGED", "UNIT_NAME_UPDATE",
							"UNIT_FACTION", "UNIT_LEVEL",
							"UNIT_CLASSIFICATION_CHANGED", "UNIT_PORTRAIT_CHANGED",
							"UNIT_PORTRAIT_UPDATE", "PET_BAR_UPDATE", "UNIT_PET", 
							"UNIT_COMBAT", "UNIT_HAPPINESS",
					}
					
CreateFrame("GameTooltip", "AuraUnitFrames_Tooltip", UIParent, "GameTooltipTemplate")

function AuraUnitFrames_OnLoad(self)
	local unitid = self:GetAttribute("unit")
	
	self:SetAttribute("type1", "target")
	
	if unitid == "player" then
		self.showmenu = function()
			ToggleDropDownMenu(1, nil, AuraUnitFrames_PlayerDropDownMenu, self)
		end
	elseif unitid == "target" then
		self.showmenu = function()
			ToggleDropDownMenu(1, nil, AuraUnitFrames_TargetDropDownMenu, self)
		end
	elseif unitid == "focus" then
		self.showmenu = function()
			ToggleDropDownMenu(1, nil, AuraUnitFrames_FocusDropDownMenu, self)
		end
	elseif unitid == "pet" then
		self.showmenu = function()
			ToggleDropDownMenu(1, nil, AuraUnitFrames_PlayerPetDropDownMenu, self)
		end
	else 
		self.showmenu = function()
			ToggleDropDownMenu(1, nil, AuraUnitFrames_PartyPetDropDownMenu, self)
		end
	end
	
	AuraUnitFrames_SetChildren(self , unitid)
	
	
	SecureUnitButton_OnLoad(self, unitid, self.showmenu)

	
	name = (self:GetName()):match("_%a+")
	--set up the buff frames
	buffframe = CreateFrame("Frame", "AuraUnitFrames"..name.."BuffFrame", self, "AuraUnitFrames_BuffFrameTemplate")
	--must call the setupvalues method otherwise we're in trouble
	AuraUnitFramesBuffs_SetUpValues(buffframe, unitid)
	
	AuraUnitFrames_DisableFrame(PlayerFrame)
	AuraUnitFrames_DisableFrame(TargetFrame)
	AuraUnitFrames_DisableFrame(PetFrame)
	
--Set up the statusbar events, should be listening for all that will effect any statusbar. Wont matter how many we add
--As a mana bar will only ever look for rage/energy/rage events not hp ones
	for i,event in ipairs(AUF_Events) do 
		self:RegisterEvent(event)   
	end	
	
	--get the frames name and set up all its shortcuts
	self.healthbar = _G[self:GetName().."_HealthBar"]
	self.manabar = _G[self:GetName().."_ManaBar"]
	self.healthbartext = _G[self.healthbar:GetName().."_Text"]
	self.manabartext = _G[self.manabar:GetName().."_Text"]
	--self.healthbartexture = _G[self:GetName().."_HealthText"]
	self.manabartexture = _G[self:GetName().."_ManaText"]
	self.unitnametext = _G[self:GetName().."_Name_Text"]
	self.unitleveltext = _G[self:GetName().."_Level_Text"]
	self.pvpicon = _G[self:GetName().."_PvP_Icon"]
	self.portrait = _G[self:GetName().."_Portrait_Picture"]
	self.hitindicator = _G[self:GetName().."_HitIndicator_Overlay"]

	CombatFeedback_Initialize(self.hitindicator, 20)
	
	--if this is the target frame register this frame to watch for target changes
	if unitid == "target" or unitid =="focus" then
		RegisterUnitWatch(self)
	end
	
	if unitid == "pet" or name:find("PartyPet") then
		self:SetScale(.75)
		RegisterUnitWatch(self)
		buffframe:SetScale(0.75)
		
		self.happiness = _G[self:GetName().."_Happiness"]
		self.happinesstext = _G[self.happiness:GetName().."_Texture"]
	end
	
	 --  Create the slash command.
	SlashCmdList["AuraUnitFrames"] = AuraUnitFrames_SlashHandler
	SLASH_AuraUnitFrames1 = "/AuraUnitFrames"
	SLASH_AuraUnitFrames2 = "/auraunitframes"
	SLASH_AuraUnitFrames3 = "/AUF"
	SLASH_AuraUnitFrames4 = "/auf"
		
end

function AuraUnitFrames_SlashHandler(msg)
	
	if msg:lower() == "lock" then
		AURAUNITFRAMES_FRAME_FRAMESLOCKED = true
	elseif msg:lower() == "unlock" then
		AURAUNITFRAMES_FRAME_FRAMESLOCKED = false
	end
	
end

function AuraUnitFrames_OnEvent(self, event, ...)
	
	local selfunit = self:GetAttribute("unit")
	
	if event=="ADDON_LOADED" then
		AUF_Loaded = true
	end
	
	if AUF_Loaded then
		if event=="PLAYER_ENTERING_WORLD" then
				--initialise the player bars on login
			if selfunit == "player" or selfunit == "pet" or selfunit:find("partypet") then
				AuraUnitFrames_Initilise(self,selfunit)
			end
		elseif event == "PLAYER_TARGET_CHANGED" and selfunit == "target" then
			AuraUnitFrames_TargetFocusUpdate(self, "target")
			CloseDropDownMenus()
		elseif event == "PLAYER_FOCUS_CHANGED" and selfunit == "focus" then
			AuraUnitFrames_TargetFocusUpdate(self, "focus")
			CloseDropDownMenus()
		elseif event == "UNIT_COMBAT" then
			if select(1,...) == selfunit then
				CombatFeedback_OnCombatEvent(select(2,...), select(3,...), select(4,...), select(5,...))
			end
		else
			--if arg1 = a unit then we do something otherwise do nothing
			local unit = ...
			--check that the frame to be effected is actually the current one
			if selfunit == unit then
				if event == "UNIT_PET" then
					if unit == "player" then
						AuraUnitFrames_Initilise(AuraUnitFrames_PlayerPetFrame, "pet")
					elseif unit == "party1" then
						AuraUnitFrames_Initilise(AuraUnitFrames_PartyPet1Frame, "partypet1")
					elseif unit == "party2" then
						AuraUnitFrames_Initilise(AuraUnitFrames_PartyPet2Frame, "partypet2")
					elseif unit == "party3" then
						AuraUnitFrames_Initilise(AuraUnitFrames_PartyPet3Frame, "partypet3")
					elseif unit == "party4" then
						AuraUnitFrames_Initilise(AuraUnitFrames_PartyPet4Frame, "partypet4")
					end
				elseif event == "UNIT_MAXHEALTH"  or event == "UNIT_HEALTH" then
					AuraUnitFrames_UpdateHealth(self,unit)
				elseif event == "UNIT_MANA" or event == "UNIT_ENERGY" or event == "UNIT_RAGE" or
							event == "UNIT_MAXMANA" or event == "UNIT_MAXENERGY" or event == "UNIT_MAXRAGE"  then
					AuraUnitFrames_UpdateMana(self,unit)
				elseif event == "UNIT_DISPLAYPOWER" then
					AuraUnitFrames_UpdatePowerType(self,unit)
				elseif event == "UNIT_NAME_UPDATE" then
					AuraUnitFrames_UpdateName(self,unit)
				elseif event ==	"UNIT_FACTION" then
					AuraUnitFrames_UpdateFaction(self,unit)
				elseif event == "UNIT_LEVEL" or event == "UNIT_CLASSIFICATION_CHANGED" then
					AuraUnitFrames_UpdateLevel(self, unit)
					--build classification changes into the level code saves time and repeated code
				elseif event == "UNIT_PORTRAIT_CHANGED" then
					SetPortraitTexture(self.portrait, unit)
				elseif event == "PET_BAR_UPDATE" then
					AuraUnitFrames_Initilise(AuraUnitFrames_PlayerPetFrame, "pet")
				elseif event == "UNIT_PORTRAIT_UPDATE" then
					SetPortraitTexture(self.portrait, unit)
				elseif event == "UNIT_HAPPINESS"  then
					if selfunit == "pet" then
						AuraUnitFrames_UpdatePet(self)
					end
				end
			end
		end
	end
end

function AuraUnitFrames_TargetFocusUpdate(self, unitid)
	if not UnitIsPlayer(unitid) then
		if UnitPowerType(unitid) == 0 then
			self.manabar:Show()
			self.manabartexture:Show()
		else
			self.manabar:Hide()
			self.manabartexture:Hide()
		end
	else 
		self.manabar:Show()
		self.manabartexture:Show()
	end

	AuraUnitFrames_Initilise(self, unitid)	
end

function AuraUnitFrames_Initilise(self, unit)
	SetPortraitTexture(self.portrait, unit)
	AuraUnitFrames_UpdateLevel(self,unit)
	AuraUnitFrames_UpdateName(self,unit)
   	AuraUnitFrames_UpdateHealth(self,unit)
   	AuraUnitFrames_UpdateFaction(self,unit)
   	--set up the players manabar colour based of class
	AuraUnitFrames_UpdatePowerType(self, unit)
	AuraUnitFrames_UpdateMana(self, unit)
	

end

function AuraUnitFrames_UpdateFaction(self, unit)
	if UnitIsPVP(unit) then
		self.pvpicon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..(UnitFactionGroup(unit) or "FFA"))
		self.pvpicon:Show()
	else
		self.pvpicon:Hide()
	end
end

function AuraUnitFrames_UpdateLevel(self,unit)
	if UnitClassification(unit) == "worldboss" then
		self.unitleveltext:SetText("Boss")
	else
		local level = UnitLevel(unit)
		-- + after the level means elite r means rare
		-- do we want to allow custom symbols?
		if UnitClassification(unit) == "rareelite" then
			level = level.."r+"
		elseif UnitClassification(unit) == "elite" then
			level = level.."+"
		elseif UnitClassification(unit) == "rare" then
			level = level.."r"
		end		
		self.unitleveltext:SetText(level)
	end
	
	
end

function AuraUnitFrames_UpdateName(self, unit)
   self.unitnametext:SetText(UnitName(unit))
end

function AuraUnitFrames_UpdateMana(self, unit)
	if UnitManaMax(unit) == 0 then
		self.manabar:Hide()
		self.manabartexture:Hide()
	else
		self.manabar:Show()
		self.manabartexture:Show()
		self.manabar:SetMinMaxValues(0, UnitManaMax(unit))
		self.manabar:SetValue(UnitMana(unit))
		local percentage = AUF_round(UnitMana(unit)/UnitManaMax(unit) * 100)
		self.manabartext:SetText(UnitMana(unit).."/"..UnitManaMax(unit).." - "..percentage.."%")
	end
end

function AuraUnitFrames_UpdateHealth(self,unit)
	self.healthbar:SetMinMaxValues(0, UnitHealthMax(unit))
	self.healthbar:SetValue(UnitHealth(unit))
	local percentage = AUF_round(UnitHealth(unit)/UnitHealthMax(unit) * 100)
	self.healthbartext:SetText(UnitHealth(unit).."/"..UnitHealthMax(unit).." - "..percentage.."%")
end

function AuraUnitFrames_UpdatePowerType(self, unit)
	local color = ManaBarColor[UnitPowerType(unit)]
	self.manabar:SetStatusBarColor(color.r, color.g, color.b)	
end

function AuraUnitFrames_UpdatePet(self)
	local happiness, damagePercentage, loyaltyRate = GetPetHappiness();
	local hasPetUI, isHunterPet = HasPetUI()
	if (not happiness or not isHunterPet) then
		if self.happiness:IsShown() then
			self.happiness:Hide()
		end
		return
	end
	self.happiness:Show();
	if ( happiness == 1 ) then
		self.happinesstext:SetTexCoord(0.375, 0.5625, 0, 0.359375)
	elseif ( happiness == 2 ) then
		self.happinesstext:SetTexCoord(0.1875, 0.375, 0, 0.359375)
	elseif ( happiness == 3 ) then
		self.happinesstext:SetTexCoord(0, 0.1875, 0, 0.359375)
	end
	self.happiness.tooltip = getglobal("PET_HAPPINESS"..happiness)
	self.happiness.tooltipDamage = format(PET_DAMAGE_PERCENTAGE, damagePercentage)
	if ( loyaltyRate < 0 ) then
		self.happiness.tooltipLoyalty = getglobal("LOSING_LOYALTY")
	elseif ( loyaltyRate > 0 ) then
		self.happiness.tooltipLoyalty = getglobal("GAINING_LOYALTY")
	else
		self.happiness.tooltipLoyalty = nil
	end
end

function AuraUnitFrames_DisableFrame(self)
	UnregisterUnitWatch(self)
	self:UnregisterAllEvents()
	self:Hide()
	self:ClearAllPoints()
	self:SetPoint("BOTTOMLEFT", UIParent, "TOPLEFT", -1000, 1000)
end

function AuraUnitFrames_SetChildren(self,unitid)
   local c = {self:GetChildren()}
   
   for i,child in pairs(c) do
		if child:GetName() ~= "AuraUnitFrames_PlayerPetFrame_Happiness" then
			child:SetAttribute("unit", unitid)
			child:SetAttribute("type1", "target")
		
			SecureUnitButton_OnLoad(child, unitid, self.showmenu)
		
			--set up the children to use the correct tooltip commands based of if they are party children or not
			if unitid == "party1" or unitid == "party2" or unitid == "party3" or unitid == "party4" then
				child:SetScript("OnEnter", AuraUnitFramesParty_OnEnter)
			else 
				child:SetScript("OnEnter", AuraUnitFrames_OnEnter)
			end
			-- hides the shown tooltip so can call default version no matter if it is party or player/target/focus
			child:SetScript("OnLeave", AuraUnitFrames_OnLeave)
		else
			return
		end
   end
      
end

function AuraUnitFrames_ToolTipOnUpdate(self)
	local unit = self:GetAttribute("unit")
	if  AuraUnitFrames_Tooltip:IsOwned(self)  then
		AuraUnitFrames_Tooltip:SetUnit(unit)
	end
end

function AuraUnitFrames_OnUpdate(self, elapsed)
	if UnitExists(self:GetAttribute("unit")) then
		CombatFeedback_OnUpdate(elapsed)
	end
end

function AuraUnitFrames_OnEnter(self)
	local unit = self:GetAttribute("unit")
	if self:GetParent():GetName() ~= "UIParent" then
		AuraUnitFrames_Tooltip:SetOwner(self:GetParent(), "ANCHOR_BOTTOMRIGHT", 25, -62)
	else 
		AuraUnitFrames_Tooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 25, -62)
	end
	AuraUnitFrames_Tooltip:SetUnit(unit)
end

function  AuraUnitFrames_OnLeave()
	 AuraUnitFrames_Tooltip:Hide()
end

function  AuraUnitFrames_OnDragStart(self)	
	if not ( AURAUNITFRAMES_FRAME_FRAMESLOCKED ) then
		self:StartMoving()
		self.isMoving = true
	end
end

function AuraUnitFrames_OnDragStop(self)
	if self.isMoving then
		self:StopMovingOrSizing()
	end
end

--function for rounding numbers for percentages
function AUF_round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) /mult
end