﻿-- Author      : Alistair
-- Create Date : 7/25/2008 3:06:23 PM

CreateFrame("GameTooltip", "AuraUnitFramesBuff_Tooltip", UIParent, "GameTooltipTemplate")

function AuraUnitFramesBuffs_OnLoad(self)
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED")
	self.buffframes = {}
	self.debuffframes = {}
end

function AuraUnitFramesBuffs_SetUpValues(self, unitid, buffs_per_row, buff_spacing, buff_type_offset_x, buff_type_offset_y )
	self:SetAttribute("unit", unitid)
	self.buffs_per_row = buffs_per_row or 8
	self.buff_spacing = buff_spacing or 2
	self.buff_type_offset_x = buff_type_offset_x or 7 
	self.buff_type_offset_y = buff_type_offset_y or -7
   
end

function AuraUnitFramesBuffs_OnEvent(self, event, unit)	
	-- When the player enters the world and the frame is the playerbuffframe then check and show all
	--buffs and debuffs on the player
	unitid = self:GetAttribute("unit")
	
	if event == "PLAYER_ENTERING_WORLD" then
		local AnchorBuff = AuraUnitFramesBuffs_UpdateBuffs(self)
		local AnchorDebuff = AuraUnitFramesBuffs_UpdateDebuffs(self)
		AuraUnitFramesBuffs_UpdateAnchors(self.buffframes[1], self.debuffframes[1], AnchorBuff, self)	
	end
	
	--Deal with the player changing targets or the targets buffs/debuffs changing and update the
	--target frame as needed
	--also do the same for the focus frame
	if (event == "PLAYER_TARGET_CHANGED" and unitid == "target" ) or
		(event == "PLAYER_FOCUS_CHANGED" and unitid == "focus" ) or
		(event == "UNIT_AURA" and unit == "target" and unitid == "target" ) or
		(event == "UNIT_AURA" and unit == "focus" and unitid == "focus" ) then
			
		local AnchorBuff = AuraUnitFramesBuffs_UpdateBuffs(self)
		local AnchorDebuff = AuraUnitFramesBuffs_UpdateDebuffs(self)
		
		--if the target is friendly buffs on top, otherwise debuffs on top	
		if UnitIsFriend("player",unitid) then
			AuraUnitFramesBuffs_UpdateAnchors(self.buffframes[1], self.debuffframes[1], AnchorBuff, self)
		else
			AuraUnitFramesBuffs_UpdateAnchors(self.debuffframes[1], self.buffframes[1], AnchorDebuff, self)
		end
	
	--else update the frames based of the unit attribute (i.e.) party2 fires a "UNIT_AURA" event make sure we only
	--adjust player2's frame
	--due to the fact that both target and party2 will fire an "UNIT_AURA" event if the player is targeting party2
	-- both the target and the party2 frame will update buff/debuffs	
	elseif (event == "UNIT_AURA") then
		if unitid == unit then
			local AnchorBuff = AuraUnitFramesBuffs_UpdateBuffs(self)
			local AnchorDebuff = AuraUnitFramesBuffs_UpdateDebuffs(self)
			AuraUnitFramesBuffs_UpdateAnchors(self.buffframes[1], self.debuffframes[1], AnchorBuff, self)
		end
	end

end

function AuraUnitFramesBuffs_UpdateAnchors(firstFrame, secondFrame, anchorTo, self)
	if firstFrame and firstFrame:IsShown() then
		firstFrame:SetPoint("TOPLEFT", 	self)
		if secondFrame and secondFrame:IsShown() then
			secondFrame:SetPoint("TOPLEFT", anchorTo, "BOTTOMLEFT", 0, self.buff_type_offset_y)
		end
	elseif secondFrame and secondFrame:IsShown() then
		secondFrame:SetPoint("TOPLEFT", self)  
	end
end

function AuraUnitFramesBuffs_CreateBuff(self, index)
	return AuraUnitFramesBuffs_CreateBuffBase(self,
	  index,
	  self.buffframes,
	  self:GetName().."Buff"..index,
	  "AuraUnitFrames_BuffTemplate"
	)
end

function AuraUnitFramesBuffs_CreateDebuff(self, index)
	return AuraUnitFramesBuffs_CreateBuffBase(self,
	  index,
	  self.debuffframes,
	  self:GetName().."Debuff"..index,
	  "AuraUnitFrames_DebuffTemplate"
	)
end

function AuraUnitFramesBuffs_CreateBuffBase(self, index, frames, name, template)
	local frame = CreateFrame("Button", name, self, template)
		
	frame.icon = _G[name.."Icon"]
	frame.cooldown = _G[name.."Cooldown"]
	frame.count = _G[name.."Count"]
	frame.border = _G[name.."Border"]
	frame.id = index
	frame.unit = self:GetAttribute("unit")
	
	local relativeTo, relativePoint, offsetX, offSetY
	if index == 1 then
		--use default (empty) values
	elseif index % self.buffs_per_row == 1 then
		relativeTo = frames[index - self.buffs_per_row]
		relativePoint = "BOTTOMLEFT"
		offsetX = 0
		offsetY = - self.buff_spacing
	else
		relativeTo = frames[index - 1]
		relativePoint = "TOPRIGHT"
		offsetX = self.buff_spacing
		offsetY = 0
	end
	
	frame:SetPoint("TOPLEFT", relativeTo, relativePoint, offSetX, offSetY)
	frames[index] = frame
	return frame
end

function AuraUnitFramesBuffs_UpdateBuffs(self)
	local unit = self:GetAttribute("unit")
	return AuraUnitFramesBuffs_UpdateBuffsBase(
				self,
				function(index)
					local icon, count, duration, timeLeft = select(3, UnitBuff(unit, index))
					return icon, count, duration, timeLeft
				end,
				self.buffframes,
				"CreateBuff"
		)
end

function AuraUnitFramesBuffs_UpdateDebuffs(self)
   	local unit = self:GetAttribute("unit")
	return AuraUnitFramesBuffs_UpdateBuffsBase(
				self,
				function(index)
					local icon, count, debuffType, duration, timeLeft = select(3, UnitDebuff(unit, index))
				    return icon, count, duration, timeLeft, debuffType
				end,
				self.debuffframes,
				"CreateDebuff"
		)
end

function AuraUnitFramesBuffs_UpdateBuffsBase(self, BuffFunc, frames, method)
	local index = 1
	local icon, count, duration, timeLeft, debuffType = BuffFunc(index)
	local anchorBuff
   
	while icon do
		local buff
		if method == "CreateBuff" then
			buff = frames[index] or AuraUnitFramesBuffs_CreateBuff(self, index)
		else
			buff = frames[index] or AuraUnitFramesBuffs_CreateDebuff(self, index)
		end
		buff:Show()
		
		if index % self.buffs_per_row ==1 then
			anchorBuff = buff
		end 	
		
		buff.icon:SetTexture(icon)
		
		if count > 1 then
			buff.count:SetText(count)
			buff.count:Show()
		else
			buff.count:Hide()
		end
		
		if duration and duration > 0 then
			local startTime = GetTime() - (duration - timeLeft)
			buff.cooldown:SetCooldown(startTime, duration)
			buff.cooldown:Show()
		else
			buff.cooldown:Hide()
		end
		
		if buff.border then
			local color
			if debuffType then
				color = DebuffTypeColor[debuffType]
			else
				color = DebuffTypeColor["none"]
			end
			buff.border:SetVertexColor(color.r, color.g, color.b)
		end
		
		index = index + 1
		icon, count, duration, timeLeft, debuffType = BuffFunc(index)
	end
	
	for i = index, #frames do
		frames[i]:Hide()
	end
	return anchorBuff
end

function AuraUnitFramesBuffs_OnUpdate(self)
	if self:GetAttribute("type") == "buff" then
		if AuraUnitFramesBuff_Tooltip:IsOwned(self)  then
			AuraUnitFramesBuff_Tooltip:SetUnitBuff(self.unit, self.id)
		end
	else
		if  AuraUnitFramesBuff_Tooltip:IsOwned(self)  then
			AuraUnitFramesBuff_Tooltip:SetUnitDebuff(self.unit, self.id)
		end
   end
end

function AuraUnitFramesBuffs_OnEnter(self)
	AuraUnitFramesBuff_Tooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 15, -25)
	if self:GetAttribute("type") == "buff" then
       AuraUnitFramesBuff_Tooltip:SetUnitBuff(self.unit, self.id)
    else
       AuraUnitFramesBuff_Tooltip:SetUnitDebuff(self.unit, self.id)
    end
end

function  AuraUnitFramesBuffs_OnLeave()
	 AuraUnitFramesBuff_Tooltip:Hide()
end